unit w_DelphiDabblerSwag;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Classes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Vcl.ExtCtrls,
  System.JSON,
  System.Generics.Collections;

const
  WM_Init = 4711;

type
  TIdTitleObj = class
  private
    FId: string;
    FTitle: string;
  public
    constructor Create(const _Id, _Title: string);
    property Id: string read FId write FId;
    property Title: string read FTitle write FTitle;
  end;

type
  Tf_DelphiDabblerSwag = class(TForm)
    m_Result: TMemo;
    lb_Categories: TListBox;
    lb_Snippets: TListBox;
    m_Snippet: TMemo;
    procedure lb_CategoriesClick(Sender: TObject);
    procedure lb_SnippetsClick(Sender: TObject);
  private
    FCategories: TObjectList<TIdTitleObj>;
    FSnippets: TObjectList<TIdTitleObj>;
    procedure WmGetCategories(var _Msg: TMessage); message WM_Init;
    procedure doRequest(const _Command: string; out _Response: TJSONObject);
    procedure RequestCategories(_Categories: TObjectList<TIdTitleObj>);
    function ReqiestSnippetCount(const _CategoryId: string): Integer;
    procedure RequestSnippets(const _CategoryId: string; _Snippets: TObjectList<TIdTitleObj>);
    procedure RequestSnippetText(const _SnippetId: string; out _Text: string);
  public
    constructor Create(_Owner: TComponent); override;
    destructor Destroy; override;
  end;

var
  f_DelphiDabblerSwag: Tf_DelphiDabblerSwag;

implementation

{$R *.dfm}
uses
  IdBaseComponent,
  IdComponent,
  IdTCPConnection,
  IdTCPClient,
  IdHTTP;

constructor Tf_DelphiDabblerSwag.Create(_Owner: TComponent);
begin
  inherited;
  FCategories := TObjectList<TIdTitleObj>.Create;
  FSnippets := TObjectList<TIdTitleObj>.Create();
  PostMessage(Handle, WM_Init, 0, 0);
end;

destructor Tf_DelphiDabblerSwag.Destroy;
begin
  FreeAndNil(FSnippets);
  FreeAndNil(FCategories);
  inherited;
end;

procedure Tf_DelphiDabblerSwag.doRequest(const _Command: string; out _Response: TJSONObject);
var
  HTTP: TIdHTTP;
  JSON: string;
begin
  HTTP := TIdHTTP.Create(nil);
  try
    JSON := HTTP.Get('http://swag.delphidabbler.com/api/v1/' + _Command);
//    m_Result.Lines.text := JSON;
  finally
    FreeAndNil(HTTP);
  end;
  _Response := TJSONObject.ParseJSONValue(JSON) as TJSONObject;
end;

function Tf_DelphiDabblerSwag.ReqiestSnippetCount(const _CategoryId: string): Integer;
var
  JObj: TJSONObject;
  SnippetCountStr: string;
begin
  doRequest('snippet-count/' + _CategoryId, JObj);
  try
    m_Result.Lines.Add('status: ' + JObj.GetValue('status').Value);
    m_Result.Lines.Add('command: ' + JObj.GetValue('command').Value);
    SnippetCountStr := JObj.GetValue('snippetCount').Value;
    m_Result.Lines.Add('snippetCount: ' + SnippetCountStr);
    Result := strToInt(SnippetCountStr);
  finally
    FreeAndNil(JObj);
  end;
end;

procedure Tf_DelphiDabblerSwag.RequestSnippets(const _CategoryId: string; _Snippets: TObjectList<TIdTitleObj>);
var
  JObj: TJSONObject;
  i: Integer;
  JSnippets: TJSONArray;
  JSnippet: TJSONObject;
  Snippet: TIdTitleObj;
begin
  _Snippets.Clear;
  doRequest('snippets/' + _CategoryId + '?fields=id,title', JObj);
  m_Result.Lines.Add('status: ' + JObj.GetValue('status').Value);
  m_Result.Lines.Add('command: ' + JObj.GetValue('command').Value);
  JSnippets := JObj.Get('snippets').JsonValue as TJSONArray;
  for i := 0 to JSnippets.Count - 1 do begin
    JSnippet := JSnippets.Items[i] as TJSONObject;
    Snippet := TIdTitleObj.Create(JSnippet.GetValue('id').Value, JSnippet.GetValue('title').Value);
//    m_Result.Lines.Add(Format('Snippet: (ID: %s, Title: %s', [JSnippet.GetValue('id').Value, JSnippet.GetValue('title').Value]));
    _Snippets.Add(Snippet);
  end;
end;

procedure Tf_DelphiDabblerSwag.lb_CategoriesClick(Sender: TObject);
var
  Idx: Integer;
  Category: TIdTitleObj;
  i: Integer;
begin
  Idx := lb_Categories.ItemIndex;
  if Idx = -1 then
    Exit;

  Category := lb_Categories.Items.Objects[Idx] as TIdTitleObj;
  RequestSnippets(Category.Id, FSnippets);
  lb_Snippets.Items.BeginUpdate;
  try
    lb_Snippets.Items.Clear;
    for i := 0 to FSnippets.Count - 1 do
      lb_Snippets.Items.AddObject(FSnippets[i].Title, FSnippets[i]);
  finally
    lb_Snippets.Items.EndUpdate;
  end;
end;

procedure Tf_DelphiDabblerSwag.RequestSnippetText(const _SnippetId: string; out _Text: string);
var
  JObj: TJSONObject;
  JSnippet: TJSONObject;
begin
  doRequest('snippet/' + _SnippetId + '?fields=source_code', JObj);
  try
    m_Result.Lines.Add('status: ' + JObj.GetValue('status').Value);
    m_Result.Lines.Add('command: ' + JObj.GetValue('command').Value);
    JSnippet := JObj.GetValue('snippet') as TJSONObject;
    m_Snippet.Lines.Text := JSnippet.GetValue('source_code').Value;
  finally
    FreeAndNil(JObj);
  end;
end;

procedure Tf_DelphiDabblerSwag.lb_SnippetsClick(Sender: TObject);
var
  SourceCode: string;
  Snippet: TIdTitleObj;
  Idx: Integer;
begin
  Idx := lb_Snippets.ItemIndex;
  if Idx = -1 then
    Exit;
  Snippet := lb_Snippets.Items.Objects[Idx] as TIdTitleObj;
  RequestSnippetText(Snippet.Id, SourceCode);
end;

procedure Tf_DelphiDabblerSwag.RequestCategories(_Categories: TObjectList<TIdTitleObj>);
var
  JObj: TJSONObject;
  JCategories: TJSONArray;
  i: Integer;
  JCategory: TJSONObject;
  Category: TIdTitleObj;
begin
  doRequest('categories', JObj);
  try
    m_Result.Lines.Add('status: ' + JObj.GetValue('status').Value);
    m_Result.Lines.Add('command: ' + JObj.GetValue('command').Value);
    JCategories := JObj.Get('categories').JsonValue as TJSONArray;
    for i := 0 to JCategories.Count - 1 do begin
      JCategory := JCategories.Items[i] as TJSONObject;
//        m_Result.Lines.Add(Format('Category: (ID: %s, Title: %s', [JCategory.GetValue('id').Value, JCategory.GetValue('title').Value]));
      Category := TIdTitleObj.Create(JCategory.GetValue('id').Value, JCategory.GetValue('title').Value);
      _Categories.Add(Category);
    end;
  finally
    FreeAndNil(JObj);
  end;
end;

procedure Tf_DelphiDabblerSwag.WmGetCategories(var _Msg: TMessage);
var
  i: Integer;
  Category: TIdTitleObj;
begin
  RequestCategories(FCategories);
  for i := 0 to FCategories.Count - 1 do begin
    Category := FCategories[i];
    lb_Categories.Items.AddObject(Category.Title, Category);
//      m_Result.Lines.Add(Format('Category: (ID: %s, Title: %s)', [Category.Id, Category.Title]));
  end;
end;

{ TIdTitleObj }

constructor TIdTitleObj.Create(const _Id, _Title: string);
begin
  inherited Create;
  FId := _Id;
  FTitle := _Title;
end;

end.

